#ifndef DUNE_CODEGEN_MATRIXFREE_HH
#define DUNE_CODEGEN_MATRIXFREE_HH

#include <iostream>

#include <dune/pdelab/backend/istl.hh>

#include <dune/codegen/matrixfreeistlbackends.hh>
#include <dune/codegen/matrixfreenewton.hh>

namespace Dune{
  namespace PDELab{

    template <typename GO, typename V>
    void solveMatrixFree(GO& go, V& x ){
      using ISTLOnTheFlyOperator = Dune::PDELab::OnTheFlyOperator<V,V,GO>;
      ISTLOnTheFlyOperator opb(go);
      Dune::Richardson<V,V> richardson(1.0);
      Dune::BiCGSTABSolver<V> solverb(opb,richardson,1E-10,5000,2);
      Dune::InverseOperatorResult stat;
      // evaluate residual w.r.t initial guess
      using TrialGridFunctionSpace = typename GO::Traits::TrialGridFunctionSpace;
      using W = Dune::PDELab::Backend::Vector<TrialGridFunctionSpace,typename V::ElementType>;
      W r(go.testGridFunctionSpace(),0.0);
      go.residual(x,r);
      // solve the jacobian system
      V z(go.trialGridFunctionSpace(),0.0);
      solverb.apply(z,r,stat);
      x -= z;
    }


    template <typename GO, typename V>
    void solveNonlinearMatrixFree(GO& go, V& x ){
      // Matrix free operator
      using ISTLOnTheFlyOperator = Dune::PDELab::NonlinearOnTheFlyOperator<V,V,GO>;
      ISTLOnTheFlyOperator op(go);

      // Matrix free linear solver
      using LinearSolver = ISTLBackend_SEQ_MatrixFree_Richardson<ISTLOnTheFlyOperator, Dune::BiCGSTABSolver>;
      LinearSolver ls(op);

      // Solve nonlinear system with matrix free newton
      using SNP = MatrixFreeNewton<GO, LinearSolver, V>;
      SNP snp(go, x, ls);
      snp.apply();
    }

  } // namespace PDELab
} // namespace Dune

#endif
