#include "config.h"

#define MAX_VECTOR_SIZE 512

#include<dune/codegen/common/vectorclass.hh>
#include<dune/codegen/sumfact/transposereg.hh>

#include<array>
#include<iostream>
#include<utility>


template<std::size_t ...S>
void apply_function(std::array<VECTYPE, M>& data, std::index_sequence<S...>)
{
  transpose_reg(std::get<S>(data) ...);
}


void print(const std::array<VECTYPE, M>& data)
{
  for(int i=0; i<M; ++i)
  {
    for(int j=0; j<N; ++j)
      std::cout << data[i].extract(j) << " ";
    std::cout << std::endl;
  }
}


int main()
{
  // Setup data
  std::array<VECTYPE, M> testdata;
  int data = 0;
  for(int i=0; i<M; ++i)
    for(int j=0; j<N; ++j, ++data)
      testdata[i].insert(j, data);

  std::array<VECTYPE, M> originaldata(testdata);

  // Print the test data once
  std::cout << "Before transposition:" << std::endl;
  print(testdata);

  // Apply the transpose function
  apply_function(testdata, std::make_index_sequence<M>{});

  // And print the test data once more
  std::cout << "After transposition:" << std::endl;
  print(testdata);

  int result = 0;
  for(int i=0; i<M; ++i)
    for(int j=0; j<N; ++j)
    {
      // Dirty handcoding of the generic permutation
      auto NM = (int)N / (int)M;
      auto linear = i * N + j;
      auto block = linear / NM;
      auto newblock = (block % M) * M + (block / M);
      auto newlinear = newblock * NM + (linear % NM);
      auto newj = newlinear % N;
      auto newi = newlinear / N;

      if(testdata[newi].extract(newj) != originaldata[i].extract(j))
      {
	std::cout << "Test failure in transpose: index (" << i << "," << j << ") in original data does not match index (" << newi << "," << newj << ") in new data." << std::endl;
	++result;
      }
    }

  return result;
}
